﻿/*----------------------------------------------------------------
//  <copyright file="DemoService.cs" company="三擎科技信息（常州）有限责任公司">
//      Copyright © 2020-2022 SANQING Corporation, All rights reserved.
//  </copyright>
//  <site>https://gitee.com/eacloud</site>
//  <last-editor>Seon.Hu</last-editor>
//  <last-date>2022-02-19 14:57</last-date>
//----------------------------------------------------------------*/

using Microsoft.Extensions.Logging;
using System;
using System.Linq;
using System.Threading.Tasks;
using DemoEntity = EaCloud.Template.Pack.Demo.Core.Entities.DemoEntity;
using Microsoft.Extensions.DependencyInjection;
using EaCloud;
using EaCloud.Data;
using EaCloud.Template.Pack.Demo.Core.Dtos;
using EaCloud.Entity;
using EaCloud.Template.Pack.Demo.Core.Entities;
using System.Data;
using System.Collections.Generic;
using System.Data.Common;

namespace EaCloud.Template.Pack.Demo.Core
{
    #region "业务实现：Demo服务"
    /// <summary>
    /// 业务实现：Demo服务
    /// </summary>
    public class DemoService : IDemoService
    {
        #region "属性"
        #region "获取 服务提供者对象"
        /// <summary>
        /// 获取 服务提供者对象
        /// </summary>
        protected IServiceProvider ServiceProvider { get; }
        #endregion
        #region "获取 日志对象"
        /// <summary>
        /// 获取 日志对象
        /// </summary>
        protected ILogger Logger => ServiceProvider.GetLogger(GetType());
        #endregion

        #region "获取 Demo存储功能"
        /// <summary>
        /// 获取 Demo存储功能
        /// </summary>
        protected IDemoStore DemoStore => ServiceProvider.GetService<IDemoStore>();
        #endregion

        #region "获取 Demo实体查询数据集"
        /// <summary>
        /// 获取 Demo实体查询数据集
        /// </summary>
        public IQueryable<DemoEntity> Demos => DemoStore.Demos;
        #endregion

        #endregion

        #region "构造函数"
        #region "初始化一个业务实现：Demo服务的新实例"
        /// <summary>
        /// 初始化一个业务实现：Demo服务（<see cref="DemoService"/>）的新实例
        /// </summary>
        /// <param name="provider">服务提供者</param>
        public DemoService(IServiceProvider provider)
        {
            ServiceProvider = provider;
        }
        #endregion

        #endregion

        #region "方法"
        #region "新增"
        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="dtos">DTO信息</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult> Create(params DemoInputDto[] dtos)
        {
            return await DemoStore.InsertByDtoAsync(dtos);
        }
        #endregion
        #region "更新"
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="dtos">DTO信息</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult> Update(params DemoInputDto[] dtos)
        {
            return await DemoStore.UpdateByDtoAsync(dtos);
        }
        #endregion
        #region "删除"
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="demoIds">数据实体编号</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult> Delete(params long[] demoIds)
        {
            return await DemoStore.DeleteAsync(demoIds);
        }
        #endregion

        #region "根据编号查找"
        /// <summary>
        /// 根据编号查找
        /// </summary>
        /// <param name="demoId">数据实体编号</param>
        /// <returns>业务操作结果</returns>
        public virtual async Task<OperationResult<DemoOutputDto>> FindById(long demoId)
        {
            Check.GreaterThan(demoId, nameof(demoId), 0);

            var demoOutputDto = Demos.Where(o => o.Id.Equals(demoId)).ToOutput<DemoEntity, DemoOutputDto>().FirstOrDefault();
            if (demoOutputDto != null)
            {
                return await Task.FromResult(new OperationResult<DemoOutputDto>(OperationResultType.Success, "OK", demoOutputDto));
            }

            return await Task.FromResult(new OperationResult<DemoOutputDto>(OperationResultType.NotFound, "Not Found", demoOutputDto));
        }
        #endregion

        #endregion

        #region "EF Core其他的一些用法Demo"
        // 调用已映射至具体方法的函数，https://docs.microsoft.com/zh-cn/ef/core/querying/user-defined-function-mapping
        #region "调用已映射至实体类的视图"
        /// <summary>
        /// 调用已映射至实体类的视图，用法同实体，可读不可写
        /// </summary>
        protected void UseViewMappedEntity()
        {
            IRepository<DemoView, int> demoViewRepository = ServiceProvider.GetService<IRepository<DemoView, int>>();
            var vals = demoViewRepository.Query(o => o.Id > 0).ToList();
            Console.WriteLine(vals.ToJsonString());
        }
        #endregion

        #region "使用SQL语句操作"
        /// <summary>
        /// 使用SQL语句操作，（建议用于复杂查询，不建议用于数据写入，数据写入依托EF CORE ORM处理）
        /// </summary>
        protected void UseSqlOperate()
        {
            // 获取数据上下文
            IDbContext dbContext;
            dbContext = ServiceProvider.GetService<IRepository<DemoView, int>>().DbContext;  // 方式1：获取一个指定模型的数据仓储所对应的数据上下文
            dbContext = ServiceProvider.GetDbContext<DemoView, int>();  //方式2：获取指定实体类型的上下文对象
            /* 获取一个工作单元 */
            IUnitOfWork unitOfWork = ServiceProvider.GetUnitOfWork();
            dbContext = unitOfWork.GetEntityDbContext(typeof(DemoView));    //方式3：获取指定数据实体的上下文实例
            dbContext = unitOfWork.GetEntityDbContext<DemoView, int>(); //方式4：获取指定数据上下文类型的实例
            dbContext = unitOfWork.GetDbContext(typeof(DefaultDbContext));  //方式5：获取指定类型的上下文实例。注：指定类型必须要继承自 DbContextBase，目前框架中默认 DefaultDbContext 类型。

            // 获取上下文数据库类型，可根据不同数据库类型操作
            var dbType = dbContext.GetDatabaseType();

            // 执行指定的Sql语句，返回影响行数
            dbContext.ExecuteSqlRaw(@$"SELECT GETDATE() AS [TimeId] INTO {"#TB01"}");
            dbContext.ExecuteSqlRaw(@"SELECT GETDATE() AS [TimeId] INTO {0}", "#TB01");

            // 执行指定的格式化Sql语句，返回影响行数
            var tmpName = "#TB01";
            dbContext.ExecuteSqlInterpolated($"SELECT GETDATE() AS [TimeId] INTO {tmpName}");

            // 执行指定的Sql语句并返回DataTable数据
            DataTable dataTable = dbContext.SqlQuery("SELECT GETDATE() AS [TimeId]");

            // 执行指定的Sql语句并返回指定类型数据
            IEnumerable<SqlQueryValues> queryValues = dbContext.SqlQuery<SqlQueryValues>("SELECT GETDATE() AS [TimeId]");

            // 数据库命令模式
            DbCommand dbCommand = dbContext.CreateDbCommand(out DbConnection dbConnection);
            dbCommand.CommandText = "";
            dbCommand.CommandType = CommandType.Text;
            //TODO：设置名称参数
            DbDataReader reader = dbCommand.ExecuteReader();
            //TODO：处理结果数据
            reader.Close();
            dbConnection.Close();
        }
        #endregion

        #endregion

    }
    #endregion

    /// <summary>
    /// Sql语句返回的类型数据
    /// </summary>
    public class SqlQueryValues
    {
        /// <summary>
        /// 获取或设置
        /// </summary>
        public DateTime TimeId { get; set; }
    }

}
