﻿/*----------------------------------------------------------------
//  <copyright file="DemoPack.cs" company="三擎科技信息（常州）有限责任公司">
//      Copyright © 2020-2022 SANQING Corporation, All rights reserved.
//  </copyright>
//  <site>https://gitee.com/eacloud</site>
//  <last-editor>Seon.Hu</last-editor>
//  <last-date>2021-12-22 09:05</last-date>
//----------------------------------------------------------------*/

using EaCloud;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;

namespace EaCloud.Template.Pack.Demo.Core.Options
{
    #region "Demo配置选项"
    /// <summary>
    /// Demo配置选项，
    /// 配置文件样式形如：
    ///   "Demo": {
    ///     "DemoString": "DemoString",
    ///     "DemoInt32": 123,
    ///     "DemoBoolean": true,
    ///     "DemoEnum": "Date",
    ///     "DemoStringArray": "1,2,3,4,5",
    ///     "DemoList": [
    ///       {
    ///         "DemoId": "manager",
    ///         "DemoName": "administrator"
    ///       }
    ///     ]
    ///   }
    /// </summary>
    public class DemoOptions : IDemoOptions
    {
        #region "属性"
        //受保护属性
        #region "获取 服务提供者对象"
        /// <summary>
        /// 获取 服务提供者对象
        /// </summary>
        protected IServiceProvider ServiceProvider { get; }
        #endregion
        #region "获取 日志对象"
        /// <summary>
        /// 获取 日志对象
        /// </summary>
        protected ILogger Logger => ServiceProvider.GetLogger(GetType());
        #endregion
        #region "获取 应用程序配置对象"
        /// <summary>
        /// 获取 应用程序配置对象
        /// </summary>
        protected IConfiguration Configuration => ServiceProvider.GetService<IConfiguration>();
        #endregion

        //公开的只读配置属性
        #region "获取 DemoString"
        /// <summary>
        /// 获取 DemoString
        /// </summary>
        public string DemoString { get; private set; }
        #endregion
        #region "获取 DemoInt32"
        /// <summary>
        /// 获取 DemoInt32
        /// </summary>
        public int DemoInt32 { get; private set; }
        #endregion
        #region "获取 DemoBoolean"
        /// <summary>
        /// 获取 DemoBoolean
        /// </summary>
        public bool DemoBoolean { get; private set; }
        #endregion
        #region "获取 DemoEnum"
        /// <summary>
        /// 获取 DemoEnum
        /// </summary>
        public DemoType DemoEnum { get; private set; }
        #endregion
        #region "获取 DemoStringArray"
        /// <summary>
        /// 获取 DemoStringArray
        /// </summary>
        public string[] DemoStringArray { get; private set; }
        #endregion

        #region "获取 Demo配置列表集合"
        /// <summary>
        /// 获取 Demo配置列表集合
        /// </summary>
        public IEnumerable<DemoOptionList> DemoList { get; private set; } = new List<DemoOptionList>();
        #endregion

        #endregion

        #region "构造函数"
        #region "初始化一个Demo配置选项（<see cref="DemoOptions"/>）的新实例"
        /// <summary>
        /// 初始化一个Demo配置选项（<see cref="DemoOptions"/>）的新实例
        /// </summary>
        /// <param name="provider">服务提供者</param>
        public DemoOptions(IServiceProvider provider)
        {
            ServiceProvider = provider;

            InitOptions();
        }
        #endregion

        #endregion

        #region "私有方法"
        #region "初始化配置选项信息"
        /// <summary>
        /// 初始化配置选项信息
        /// </summary>
        private void InitOptions()
        {
            var section = Configuration.GetSection("EaCloud:Demo");
            if (!section.Exists())
            {
                throw new EaCloudException("Demo配置选项不存在，请在appsetting.json配置 EaCloud:Demo 节点");
            }

            DemoString = Configuration["EaCloud:Demo:DemoString"] ?? "Default value";
            DemoInt32 = Configuration["EaCloud:Demo:DemoInt32"].CastTo(0);
            DemoBoolean = Configuration["EaCloud:Demo:DemoBoolean"].CastTo(false);
            DemoEnum = Configuration["EaCloud:Demo:DemoEnum"].CastTo(DemoType.Date);

            var demoStringArrayStr = Configuration["EaCloud:Demo:DemoStringArray"];
            DemoStringArray = demoStringArrayStr == null || demoStringArrayStr == "*" ? Array.Empty<string>() : demoStringArrayStr.Split(',');

            var demoListSection = Configuration.GetSection("EaCloud:Demo:DemoList");
            DemoList = demoListSection.Get<IEnumerable<DemoOptionList>>();

        }
        #endregion

        #endregion
    }
    #endregion

    #region "Demo配置列表"
    /// <summary>
    /// Demo配置列表
    /// </summary>
    public class DemoOptionList
    {
        #region "属性"
        #region "获取或设置 DemoId"
        /// <summary>
        /// 获取或设置 DemoId
        /// </summary>
        public string DemoId { get; set; }
        #endregion
        #region "获取或设置 DemoName"
        /// <summary>
        /// 获取或设置 DemoName
        /// </summary>
        public string DemoName { get; set; }
        #endregion

        #endregion

    }
    #endregion

}
